from fractions import Fraction
from __encontrar_N import encontrar_N
from __encontrar_N_zeta import encontrar_N_zeta
from __encontrar_zeta1Sig import encontrar_zeta1Sig
from __encontrar_zetaAproxVer import encontrar_zetaAproxVer
from __encontrar_zetaMin import encontrar_zetaMin
from __encontrar_zetaSig import encontrar_zetaSig
from __encontrar_zetaVer import encontrar_zetaVer


def constantes (bits=256, largo=28, tipo='a', tolerancia=Fraction ('15/10'), mostrar=True):

	if tipo in ('v', 'ver'):
		if mostrar: print ('# calculando parametros - optimo para verificar rapido')
		N = encontrar_N (largo, bits)
		zeta = encontrar_zetaVer (N, largo, bits, mostrar=mostrar)

	elif tipo in ('a', 'aprox'):
		if mostrar: print ('# calculando parametros - casi optimo para verificar rapido, bastante rapido para firmar')
		zeta = encontrar_zetaAproxVer (largo, bits, tolerancia, mostrar=mostrar)
		N = encontrar_N_zeta (largo, zeta, bits)

	elif tipo in ('s', 'sig'):
		if mostrar: print ('# calculando parametros - optimo para firmar rapido (modo normal)')
		zeta = encontrar_zetaSig (largo, bits, mostrar=mostrar)
		N = encontrar_N_zeta (largo, zeta, bits)

	elif tipo == '1/s':
		if mostrar: print ('# calculando parametros - optimo para firmar rapido (modo inverso)')
		zeta = encontrar_zeta1Sig (largo, bits, mostrar=mostrar)
		N = encontrar_N_zeta (largo, zeta, bits)

	elif tipo == 'zmin':
		if mostrar: print ('# calculando parametros - minimo zeta posible')
		zeta = encontrar_zetaMin (largo, bits, mostrar=mostrar)
		N = encontrar_N_zeta (largo, zeta, bits)

	elif tipo == 'zmax':
		if mostrar: print ('# calculando parametros - sin restringir zeta')
		N = encontrar_N (largo, bits)
		zeta = N

	else:
		raise ValueError ("tipo debe ser 'v', 'a', 's' o '1/s'")

	return N, zeta


# ---------------- ejemplo ----------------
if __name__ == '__main__':

	bits = 128
	largo = 64

	N, zeta = constantes (bits, largo, tipo='v', mostrar=True)
	print ('\tbits, largo = ' + str ((bits, largo)))
	print ('\tN, zeta = ' + str ((N, zeta)))

	N, zeta = constantes (bits, largo, tipo='a', mostrar=True)
	print ('\tbits, largo = ' + str ((bits, largo)))
	print ('\tN, zeta = ' + str ((N, zeta)))

	N, zeta = constantes (bits, largo, tipo='s', mostrar=True)
	print ('\tbits, largo = ' + str ((bits, largo)))
	print ('\tN, zeta = ' + str ((N, zeta)))

	N, zeta = constantes (bits, largo, tipo='1/s', mostrar=True)
	print ('\tbits, largo = ' + str ((bits, largo)))
	print ('\tN, zeta = ' + str ((N, zeta)))

